class GamingBoard extends Phaser.Scene {
    constructor(father) {
        super('GamingBoard');
        this.listRobot = [];
        this.father = father;
    }

    preload() {
        this.cameras.main.setViewport(WIDTH_WINDOW / 10, HEIGHT_WINDOW / 10, WIDTH_MAP, WIDTH_MAP);
    }

    create() {
        this.end = false;
        this.sound.add('shot');
        this.sound.add('hit');

        this.add.image(0, 0, 'background').alpha = 0.1;

        for (let i = 0; i < this.listRobot.length; i++) {
            this.chooseTarget(this.listRobot[i]);
        }

        this.listRobot[0].drawRange();

        this.events.on('resume', () => this.resume());

        this.time.addEvent({delay: 2000, callback: this.upShield, callbackScope: this});
    }

    update(time, delta) {
        super.update(time, delta);

        this.listRobot.forEach(function (robot) {
            robot.read();
        })

        for (let i = 0; i < this.listRobot.length; i++) {
            if (!this.listRobot[i].isAlive()) {
                this.listRobot.splice(i, 1);
                i -= 1;
            } else {
                if (!this.listRobot[i].haveTarget()) {
                    this.chooseTarget(this.listRobot[i]);
                } else {
                    this.listRobot[i].updateTarget();
                }
            }
        }

        if (this.listRobot.length <= 1 && !this.end) {
            this.end = true;
            this.time.addEvent({delay: 1000, callback: this.finish, callbackScope: this});
        }
    }

    finish() {
        console.log("QUOI ?");
        this.pause();
        this.father.finish();
    }

    winner() {
        return this.listRobot.length !== 0 && this.listRobot[0].name === 'myself';
    }

    pause() {
        this.scene.pause('GamingBoard');
        this.sound.pauseAll();
    }

    resume() {
        this.scene.resume('GamingBoard');
        this.sound.resumeAll();
    }

    addRobot(robot) {
        this.listRobot.push(robot);
    }

    modifyNodes(lNodes) {
        for (let i = 0; i < this.listRobot.length; i++) {
            if (this.listRobot[i].name === MYSELF) {
                this.listRobot[i].cleanNodes();
                lNodes.forEach(element => this.listRobot[0].addNode(element));
                return true;
            }
        }
        return false;
    }

    chooseTarget(robot) {
        let minDist = WIDTH_MAP * 2;
        let l = [];
        this.listRobot.forEach(function (item) {
            if (item !== robot && item.isAlive()) {
                let hypot = Math.hypot(item.x - robot.x, item.y - robot.y);
                if (minDist >= hypot) {
                    if (minDist > hypot) {
                        minDist = hypot;
                        l = [];
                    }
                    l.push(item);
                }
            }
        });
        if (this.listRobot.length > 1) {
            robot.setTarget(l[Math.floor(Math.random() * (l.length + 1))]);
        } else {
            robot.setTarget(l[0]);
        }
    }


    upShield() {
        this.listRobot.forEach(robot => robot.addShield(SHIELD_PER_SECOND));
        this.time.addEvent({delay: 1000, callback: this.upShield, callbackScope: this});
    }

}